-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- dungeontactics.lua

ENUM_MENU     = 1
ENUM_GAME     = 2
ENUM_STRATEGY = 3
ENUM_SCORES   = 4
ENUM_OPTIONS  = 5
ENUM_ABOUT    = 6
ENUM_LOAD     = 7
ENUM_EDIT     = 8
ENUM_INTRO    = 9

version = 1.6
path = "modules/dungeontactics/"
function doload( f )
	print ("[dungeontactics] loading file " .. path .. f )
	loadfile ( path .. f )()
end

-- utility objects
doload ( "statemachine.lua")     -- finite state machine 
doload ( "state.lua")            -- polymorphic base state
doload ( "pathnode.lua")         -- a* path node
doload ( "path.lua")             -- a* pathfinding
doload ( "settings.lua")         -- game settings
doload ( "_db.lua")              -- database creation script

-- game states
doload ( "state_menu.lua")       -- main menu
doload ( "state_strategy.lua")   -- metagame
doload ( "state_game.lua")       -- gameplay logic
doload ( "state_scores.lua")     -- high score list
doload ( "state_options.lua")    -- options
doload ( "state_about.lua")      -- credits / nag screen
doload ( "state_edit.lua")       -- campaign editor
doload ( "state_load.lua")       -- serialization features
doload ( "state_intro.lua")      -- intro screen

-- ui objects
doload ( "manager.lua")          -- control surface
doload ( "actions.lua")          -- unit actions menu
doload ( "history.lua")          -- text log for combat history
doload ( "summary.lua")          -- end of level summary screen
doload ( "camera.lua")           -- 3d camera
doload ( "base.lua")             -- unit deployment
doload ( "info.lua")             -- unit stat info
doload ( "inventory.lua")        -- shared item list
doload ( "alert.lua")            -- yes/no quit dialog
doload ( "specials.lua")         -- special abilities list
doload ( "support.lua")          -- support menu
doload ( "unitlist.lua")         -- unit overview
doload ( "logistics.lua")        -- town services
doload ( "selector.lua")         -- select quest
doload ( "creator.lua")          -- create character
doload ( "vendor.lua")           -- merchant system
doload ( "medical.lua")          -- heal and rez
doload ( "dialog.lua")           -- conversation system

-- game objects
doload ( "user.lua")             -- player data
doload ( "item.lua")             -- gear and equipment
doload ( "ability.lua")          -- class skills
doload ( "mob.lua")              -- moveable object
doload ( "node.lua")             -- static object
doload ( "tile.lua")             -- terrain
doload ( "map.lua")              -- playing field
doload ( "visual.lua")           -- graphical effects

-- editor objects
doload ( "edit_modules.lua")     -- add/edit campaign modules
doload ( "edit_classes.lua")     -- add/edit character classes
doload ( "edit_items.lua")       -- add/edit gear
doload ( "edit_abilities.lua")   -- add/edit special abilities
doload ( "edit_effects.lua")     -- add/edit status effects
doload ( "edit_encounters.lua")  -- add/edit encounters
doload ( "edit_tiles.lua")       -- add/edit tiles
doload ( "edit_stages.lua")      -- add/edit stages
doload ( "edit_dialog.lua")      -- add/edit lines of dialog

-- required
function init()
	print "[dungeontactics] init dungeontactics"

	-- load database
	sql = xcore.xsql( path .. "resources/tactical.db" )
	-- createDB() -- create database

	-- load settings
	-- print "[dungeontactics] load settings"
	loadSettings()
	
	-- initalize simulation
	-- print "[dungeontactics] init sim"
	sim = xcore.xsimulation ( xcore.XDT_OPENGL, screen_width, screen_height, 32, fs, sh, vsynch ) -- initalize 
	sim:setWindowCaption( "Dungeon Tactics" )
	sim:setColour( xcore.xcolour(255, 0, 0, 0) )

	-- set custom fonts
	font = sim:getBuiltInFont()	
	font_1 = sim:getFont( path .. "resources/interface/font_fixedsys_10.bmp" )
	sim:setFont( font_1,  0 ) -- default font
	sim:setFont( font_1,  1 ) -- button font
	sim:setFont( font_1,  2 ) -- UI font
	sim:setFont( font_1,  3 ) -- widget font	

	-- init sound
	-- print "[dungeontactics] new sound system"
	snd = xcore.xaudio()
	snd:clear()	

	-- print "[dungeontactics] new statemachine"
	fsm = statemachine:new() -- initalize state machine

	-- print "[dungeontactics] new states"
	menu     = state_menu:new()
	game     = state_game:new()	
	strategy = state_strategy:new()
	scores   = state_scores:new()
	options  = state_options:new()
	about    = state_about:new()
	load     = state_load:new()
	edit     = state_edit:new()
	intro    = state_intro:new()

	-- print "[dungeontactics] add states to fsm"
	fsm:addState( menu     )
	fsm:addState( game     )	
	fsm:addState( strategy )
	fsm:addState( scores   )
	fsm:addState( options  )
	fsm:addState( about    )
	fsm:addState( load     )
	fsm:addState( edit     )
	fsm:addState( intro    )
	-- print ("[dungeontactics] state count: " .. fsm:count())

	-- print "[dungeontactics] set default state"
	fsm:setDefault( ENUM_MENU )

	-- bind keys
	sim:bindkey ( "key_pressed_exit_state", xcore.XKEY_ESCAPE )
	sim:bindkey ( "key_pressed_cam_left",   xcore.XKEY_LEFT   )
	sim:bindkey ( "key_pressed_cam_right",  xcore.XKEY_RIGHT  )
	sim:bindkey ( "key_pressed_cam_up",     xcore.XKEY_UP     )
	sim:bindkey ( "key_pressed_cam_down",   xcore.XKEY_DOWN   )
	sim:bindkey ( "key_pressed_cam_in",     xcore.XKEY_PLUS   )
	sim:bindkey ( "key_pressed_cam_out",    xcore.XKEY_MINUS  )
	sim:bindkey ( "key_pressed_info_mode",  xcore.XKEY_TAB    )
	sim:bindkey ( "key_pressed_debug",      xcore.XKEY_F12    )
	sim:bindkey ( "key_pressed_cam_rotate", xcore.XKEY_SPACE  )

	sim:bindkey ( "key_pressed_move",       xcore.XKEY_M      )
	sim:bindkey ( "key_pressed_attack",     xcore.XKEY_A      )
	sim:bindkey ( "key_pressed_wait",       xcore.XKEY_W      )
	sim:bindkey ( "key_pressed_end_turn",   xcore.XKEY_E      )
	sim:bindkey ( "key_pressed_home",       xcore.XKEY_H      )
	sim:bindkey ( "key_pressed_special",    xcore.XKEY_S      )
	
	-- keydelay
	sim:setKeyDelay( 25 )
	-- load style
	loadStyle()
end

-- required
function render()
	-- print "[dungeontactics] render"
	sim:render()
end

-- required
function update()
	-- print "[dungeontactics] update"
	fsm:step()
end

-- end of file